// Copyright 2022 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package configuration

import (
	"bytes"
	"fmt"
	"io"
	"testing"
	"time"
)

type mockTestEventHandler struct {
	ChangeHook
}

func (m *mockTestEventHandler) Handle(event ChangeEvent) {
	// do nothing

}

func TestAddRemoveHook(t *testing.T) {

	config := struct {
		Host string
	}{
		Host: "localhost",
	}

	s := New(config)

	var h ChangeHook
	h = &mockTestEventHandler{}
	s.OnChange(h)

	if len(s.hooks.change) != 1 {
		t.Error("Expected 1 got ", len(s.hooks.change))
	}

	s.RemoveOnChangeHook(h)

	if len(s.hooks.change) != 0 {
		t.Error("Expected 0 got ", len(s.hooks.change))
	}

}

func TestReadmeExample(t *testing.T) {

	config := struct {
		Host string
	}{
		Host: "localhost",
	}

	s := New(config)

	closeChan := make(chan bool)

	msg := ""

	var h ChangeHook
	h = &ChangeEventHandler{
		Callback: func(event ChangeEvent) {
			log := event.Changelog
			msg = fmt.Sprintf("Change from %s to %s", log[0].From, log[0].To)
			// for Readme
			//fmt.Println(msg)
			closeChan <- true
		},
	}

	s.OnChange(h)

	c := s.Config()
	c.Host = "www.example.com"

	s.SetConfig(c)

	// Wait for change
	select {
	case <-closeChan:
	case <-time.After(time.Second):
		t.Fatalf("Timeout")
	}

	if msg != "Change from localhost to www.example.com" {
		t.Error("Expected \"Change from localhost to www.example.com\" got ", msg)
	}

}

func TestChangeOnChange(t *testing.T) {

	defaults := ConfigStruct2{
		A: "",
		B: false,
		C: nil,
		D: nil,
		F: 0,
		G: "",
	}

	s := New(defaults)
	s.SetMnemonic("test")

	var buf bytes.Buffer
	buf.WriteString("A: a\n")
	r := (io.Reader)(&buf)

	s.AddReader(r, Yaml)

	s.Import()

	if s.HasErrors() {
		t.Error("Expected not error", s.Errors())
	}

	c := s.Config()
	if c.A != "a" {
		t.Error("Expected \"a\" got ", c)
	}

	closeChan := make(chan bool)

	counter := 0

	var h ChangeHook
	h = &ChangeEventHandler{
		Callback: func(event ChangeEvent) {
			counter++
			closeChan <- true
		},
	}

	s.OnChange(h)

	c.A = "b"
	s.SetConfig(c)

	select {
	case <-closeChan:
	case <-time.After(time.Second):
		t.Fatalf("Timeout")
	}

	if counter != 1 {
		t.Error("Expected 1 got ", counter)
	}

}
